'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.filterValidate = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _immutable = require('immutable');

var _lodash = require('lodash');

var _FormCollection = require('./FormCollection');

var _FormCollection2 = _interopRequireDefault(_FormCollection);

var _createStore = require('./createStore');

var _createStore2 = _interopRequireDefault(_createStore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var PATH = 'immutable-form/';
// Form level
var RESET_FORM = PATH + 'RESET_FORM';
var CLEAR_ERRORS = PATH + 'CLEAR_ERRORS';
var ADD_ERROR = PATH + 'ADD_ERROR';
// Field level
var SET_FIELD = PATH + 'SET_FIELD';
var REMOVE_FIELD = PATH + 'REMOVE_FIELD';
var RESET_FIELD = PATH + '/RESET_FIELD';

var defaultOptions = {
  addToFormCollection: true,
  logger: false,
  store: null
};

var initialForm = (0, _immutable.Map)({
  fields: (0, _immutable.Map)(),
  errors: (0, _immutable.Stack)()
});

var initialField = (0, _immutable.Map)({
  value: '',
  errors: (0, _immutable.Stack)()
});

var createReducer = function createReducer(initialState) {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments[1];

    switch (action.type) {
      case SET_FIELD:
        {
          var _ret = function () {
            var _action$payload = action.payload,
                field = _action$payload.field,
                value = _action$payload.value,
                error = _action$payload.error;

            var path = ['fields', field];
            var nextState = state.hasIn(path) ? state : state.setIn(path, initialField);
            if (value || value === '') {
              nextState = nextState.setIn([].concat(path, ['value']), value);
            }
            if (error) {
              if (nextState.hasIn([].concat(path, ['errors']))) {
                nextState = nextState.updateIn([].concat(path, ['errors']), function (errorsStack) {
                  return errorsStack.push(error);
                });
              } else {
                nextState = nextState.setIn([].concat(path, ['errors']), _immutable.Stack.of(error));
              }
            } else if (error === null) {
              nextState = nextState.setIn([].concat(path, ['errors']), (0, _immutable.Stack)());
            }
            return {
              v: nextState
            };
          }();

          if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
        }
      case RESET_FIELD:
        {
          var _field = action.payload.field;

          return state.setIn(['fields', _field], initialField);
        }
      case REMOVE_FIELD:
        {
          var _field2 = action.payload.field;

          return state.deleteIn(['fields', _field2]);
        }
      case ADD_ERROR:
        {
          var _error = action.payload.error;

          var errorsStack = state.has('errors') ? state.get('errors').push(_error) : _immutable.Stack.of(_error);
          return state.set('errors', errorsStack);
        }
      case CLEAR_ERRORS:
        {
          return state.set('errors', (0, _immutable.Stack)());
        }
      case RESET_FORM:
        {
          return state.update(function () {
            return action.payload.initialState;
          });
        }
      default:
        break;
    }
    return state;
  };
};

var filterValidate = function filterValidate(state) {
  var fields = {};
  var form = _extends({}, state);
  var fieldValidators = {};
  var formValidators = [];
  if ((0, _lodash.has)(form, 'fields')) {
    fields = (0, _lodash.keys)(form.fields).reduce(function (res, key) {
      if ((0, _lodash.hasIn)(form, ['fields', key, 'validate'])) {
        fieldValidators[key] = Array.isArray(form.fields[key].validate) ? form.fields[key].validate : [form.fields[key].validate];
        delete form.fields[key].validate;
      }
      return _extends({}, res, _defineProperty({}, key, form.fields[key]));
    }, {});
  }
  if ((0, _lodash.has)(form, 'validate')) {
    formValidators = Array.isArray(form.validate) ? form.validate : [form.validate];
    delete form.validate;
  }
  var filteredState = _extends({}, form, {
    fields: fields
  });
  return { filteredState: filteredState, fieldValidators: fieldValidators, formValidators: formValidators };
};

exports.filterValidate = filterValidate;

var Form = function () {
  function Form(name, initialState) {
    var _this = this;

    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultOptions;

    _classCallCheck(this, Form);

    this.name = name;
    this.options = options;
    this.fieldValidators = {};
    this.formValidators = [];
    // Ensure the Form has a name
    if (!(typeof this.name === 'string') || this.name.trim().length === 0) {
      throw new Error('Form must have a name');
    }

    var middleware = [];
    // Add logging middleware
    if (this.options.logger) {
      if (require.resolve('redux-logger')) {
        // eslint-disable-next-line global-require
        var createLogger = require('redux-logger');
        middleware.push(createLogger());
      }
    }

    // If initial state is provived, need to filter and extract
    // the validation functions.
    var state = void 0;
    var tempState = (0, _lodash.cloneDeep)(initialState);
    if (!(0, _lodash.isEmpty)(tempState)) {
      (function () {
        var _filterValidate = filterValidate(tempState),
            filteredState = _filterValidate.filteredState,
            fieldValidators = _filterValidate.fieldValidators,
            formValidators = _filterValidate.formValidators;

        _this.fieldValidators = fieldValidators;
        _this.formValidators = formValidators;
        // Create the Immutable object from initial state
        var fields = !(0, _lodash.has)(filteredState, 'fields') ? (0, _immutable.Map)() : (0, _lodash.keys)(filteredState.fields).reduce(function (res, key) {
          return res.set(key, (0, _immutable.Map)({
            value: (0, _lodash.has)(filteredState.fields[key], 'value') ? filteredState.fields[key].value : '',
            errors: (0, _lodash.has)(filteredState.fields[key], 'errors') ? (0, _immutable.Stack)(filteredState.fields[key].errors) : (0, _immutable.Stack)()
          }));
        }, (0, _immutable.Map)());
        state = (0, _immutable.Map)({
          fields: fields,
          errors: (0, _lodash.has)(filteredState, 'errors') ? (0, _immutable.Stack)(filteredState.errors) : (0, _immutable.Stack)()
        });
      })();
    } else {
      state = initialForm;
    }

    this.initialState = (0, _lodash.cloneDeep)(state);

    // Keep track of form and field validation functions
    this.store = defaultOptions.store || (0, _createStore2.default)({
      reducers: {
        form: createReducer(state)
      },
      middleware: middleware
    });

    if (options.addToFormCollection) {
      _FormCollection2.default.add(this);
    }
  }

  _createClass(Form, [{
    key: 'getState',
    value: function getState() {
      return this.store.getState().get('form');
    }
  }, {
    key: 'setField',
    value: function setField(field, value, error) {
      this.store.dispatch({
        type: SET_FIELD,
        payload: {
          field: field,
          value: value,
          error: error
        }
      });
    }
  }, {
    key: 'addFieldValidator',
    value: function addFieldValidator(field, validator) {
      if (!this.fieldValidators[field]) {
        this.fieldValidators[field] = [];
      }
      this.fieldValidators[field].push(validator);
    }
  }, {
    key: 'removeFieldValidator',
    value: function removeFieldValidator(field, validator) {
      var validators = this.fieldValidators[field];
      if (validators) {
        var index = validators.indexOf(validator);
        if (index !== -1) {
          validators.splice(index, 1);
        }
      }
    }
  }, {
    key: 'getField',
    value: function getField(field) {
      return this.store.getState().getIn(['form', 'fields', field], initialField);
    }
  }, {
    key: 'removeField',
    value: function removeField(field) {
      this.store.dispatch({
        type: REMOVE_FIELD,
        payload: {
          field: field
        }
      });
    }
  }, {
    key: 'resetField',
    value: function resetField(field) {
      this.store.dispatch({
        type: RESET_FIELD,
        payload: {
          field: field
        }
      });
    }
  }, {
    key: 'addError',
    value: function addError(error) {
      this.store.dispatch({
        type: ADD_ERROR,
        payload: {
          error: error
        }
      });
    }
  }, {
    key: 'clearErrors',
    value: function clearErrors() {
      var _this2 = this;

      this.getFieldValues().forEach(function (value, key) {
        return _this2.setField(key, null, null);
      });
      this.store.dispatch({
        type: CLEAR_ERRORS
      });
    }
  }, {
    key: 'hasErrors',
    value: function hasErrors() {
      var form = this.getState();
      // Checks all the form's fields for errors and the top level form error
      return form.get('fields').reduce(function (prev, curr) {
        return prev || prev === false && curr.has('errors') && curr.get('errors').size > 0;
      }, false) || form.has('errors') && form.get('errors').size > 0;
    }
  }, {
    key: 'reset',
    value: function reset() {
      this.store.dispatch({
        type: RESET_FORM,
        payload: {
          initialState: this.initialState
        }
      });
    }
  }, {
    key: 'saveInitialState',
    value: function saveInitialState() {
      this.initialState = this.getState();
    }
  }, {
    key: 'getFields',
    value: function getFields() {
      return this.getState().get('fields', (0, _immutable.Map)());
    }
  }, {
    key: 'getFieldValues',
    value: function getFieldValues() {
      var fields = this.getState().get('fields', (0, _immutable.Map)()).map(function (value) {
        return value.has('value') ? value.get('value') : '';
      });
      return fields;
    }
  }, {
    key: 'getFieldErrors',
    value: function getFieldErrors() {
      var errors = this.getState().get('fields', (0, _immutable.Map)()).map(function (value) {
        return value.has('errors') ? value.get('errors') : (0, _immutable.Stack)();
      });
      return errors;
    }
  }, {
    key: 'getFormErrors',
    value: function getFormErrors() {
      var errors = this.getState().get('errors', (0, _immutable.Stack)());
      return errors;
    }
  }, {
    key: 'validate',
    value: function validate() {
      var _this3 = this;

      // Run form level validators
      this.formValidators.forEach(function (validator) {
        var res = validator(_this3.getState(), {
          form: _this3
        });
        if ((0, _lodash.isString)(res)) {
          _this3.addError(res);
        }
      });

      // Run field level validators
      (0, _lodash.keys)(this.fieldValidators).forEach(function (key) {
        _this3.fieldValidators[key].forEach(function (validator) {
          var value = _this3.getField(key).get('value', '');
          var res = validator(value, {
            key: key,
            form: _this3
          });
          if ((0, _lodash.isString)(res)) {
            _this3.setField(key, null, res);
          }
        });
      });

      return !this.hasErrors();
    }
  }, {
    key: 'setLoad',
    value: function setLoad(promise) {
      this.loadPromise = typeof promise === 'function' ? promise : function () {
        return promise;
      };
      return this;
    }
  }, {
    key: 'load',
    value: function () {
      var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(props) {
        var promise;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.loadPromise({ props: props, form: this });

              case 2:
                promise = _context.sent;
                return _context.abrupt('return', promise);

              case 4:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function load(_x3) {
        return _ref.apply(this, arguments);
      }

      return load;
    }()
  }, {
    key: 'setSubmit',
    value: function setSubmit(promise) {
      this.submitPromise = typeof promise === 'function' ? promise : function () {
        return promise;
      };
      return this;
    }
  }, {
    key: 'setOnSuccess',
    value: function setOnSuccess(func) {
      this.onSuccess = func;
      return this;
    }
  }, {
    key: 'setOnFailure',
    value: function setOnFailure(func) {
      this.onFailure = func;
      return this;
    }
  }, {
    key: 'submit',
    value: function submit(props) {
      var _this4 = this;

      this.clearErrors();
      var submitPromise = this.submitPromise ? this.submitPromise : function () {
        return Promise.resolve();
      };
      return new Promise(function (resolve, reject) {
        if (_this4.validate()) {
          submitPromise({ props: props, form: _this4 }).then(function (res) {
            if (_this4.onSuccess) {
              _this4.onSuccess(res, _this4);
            }
            resolve(res);
          }).catch(function (err) {
            if (_this4.onFailure) {
              _this4.onFailure(err, _this4);
            }
            reject(err);
          });
        } else {
          reject('Validation failed');
        }
      });
    }
  }]);

  return Form;
}();

exports.default = Form;